/*
Copyritht (C)2006 George Ang <gnap.an [AT] gmail.com> 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include<inttypes.h>
#include <string>
#include <errno.h>
#include <kademlia/kadlistener.hpp>
#include <kademlia/contact.hpp>
#include <kademlia/utils/uint160.hpp>
#include "UDPListener.h"
#include "dhtdebug.h"

#ifndef DHTLISTENER_HPP
#define DHTLISTENER_HPP

using namespace kademlia;

class CDHTListener : public kad_listener<uint160, dhtdebug>
{
    public:
	typedef uint160 uint_type;
	typedef dhtdebug debugger;
	typedef contact<uint_type> contact_type;

	CDHTListener(IBTTask *);

//	CDHTListener(std::string const& host, std::string const& port);

	~CDHTListener();

 	void setBTTask(IBTTask* task);

	IBTTask* getBTTask();		


	bool  add_contact (contact_type *contact_ptr, bool keep = false);
	//void add_contacts (contact_list *contact_list_ptr);
	//void add_search_results(uint32_t search_id, std::string context); 

	bool is_connected();
	
	void send_HELLO(std::string const& client, int port);
	void send_HELLO(contact_type *contact_ptr);

	void send_BOOTSTRAP_REQUEST(contact_type *contact_ptr);

	void send_BOOSTRAP_REQUEST(std::string const& host, std::string const& port);

	void send_NODE_LOOKUP_REQUEST(uint_type const& target_id,
			uint_type const& session_id,
			contact_type *contact_ptr);

	void send_STORE_VALUE(uint_type const& target_id,
			std::string const& content,
			std::string const& arguments,
			contact_type* contact_ptr);

	void send_SEARCH_REQUEST(uint_type const& target_id,
			uint_type const& session_id,
			std::string const& arguments,
			contact_type* contact_ptr);

	void process_recv_packet(char* buf, int len, std::string const& client, std::string const& port);

	//Incoming protocol data processing, NOT HAVE TO be implemented. 
	void process_BOOTSTRAP_REQUEST (contact_type* contact_ptr);

	void process_BOOTSTRAP_REPLY (const char* protocol_data, size_t len, contact_type* contact_ptr);

	void process_HELLO(contact_type* contact_ptr);

	void process_HELLO_REPLY(contact_type* contact_ptr);

	void process_NODELOOKUP_REQUEST(const char* protocol_data, size_t len, contact_type* contact_ptr);

	void process_NODELOOKUP_REPLY(const char* protocol_data, size_t len, contact_type* contact_ptr);

	void process_SEARCH_REQUEST(const char* protocol_data, size_t len, contact_type* contact_ptr);

	void process_SEARCH_REPLY(const char* protocol_data, size_t len, contact_type* contact_ptr);

	void process_STOREVALUE_REQUEST(const char* protocol_data, size_t len, contact_type* contact_ptr);

	void processPkg(const char* buf, int len, std::string const& client, const int port);
private:
	CUDPListener *m_connection;
	IBTTask* _task;
};

#endif //DHTLISTENER_HPP

