 /*
 * Author:	George Ang
 * Create data:	2007-02-11 23:25
 */

#include <string>
#include <unistd.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "UDPListener.h"
#include "DHTListener.h"
#include "log.h"
#include "dhtdebug.h"

#define RECV_BUFFER_SIZE		8*1024
CUDPListener::CUDPListener()
: _natTimerID(0)
{
}

CUDPListener::~CUDPListener()
{
}

void CUDPListener::setBTTask(IBTTask* task)
{
	_task = task;
}

void CUDPListener::setDHTListener(CDHTListener * listener)
{
	m_listener = listener;
}
IBTTask* CUDPListener::getBTTask()
{
	return _task;
}

bool CUDPListener::start()
{
	for(unsigned short i = 8681; i< 8690; ++i)
	{
		createUDPSocket();
		
		if(CSocket::bind(NULL, i))
		{
			_port = i;
//			CSocket::listen();
			CSocket::setReactor(_task->getSocketReactor());
			maskRead(true);
			
			_task->getUPnpNat()->addPortMapping(_port, "UDP");
			
			return true;
		}
	}
	return false;
}

void CUDPListener::stop()
{
	if(_natTimerID != 0)
	{
		_task->getSocketReactor()->removeTimer(_natTimerID);
	}
	
	maskRead(false);
	CSocket::setReactor(NULL);
	CSocket::close();
}

int CUDPListener::handleRead()
{
	char* buf = new char[RECV_BUFFER_SIZE];
	int status;
	struct sockaddr cliaddr;
	memset(&cliaddr, 0, sizeof(cliaddr));
	socklen_t cliaddr_len = sizeof(cliaddr);
	std::string host;
	int port;

	status = recvfrom(getHandle(), buf, RECV_BUFFER_SIZE, 0, &cliaddr, &cliaddr_len);

	if ( status < 0)
	{
		dhtdebug::send(NETWORK,"error on recvfrom(), %d", status);
	}


	host = std::string(inet_ntoa(((struct sockaddr_in *)&cliaddr)->sin_addr));
	port = ntohs(((struct sockaddr_in *)&cliaddr)->sin_port);
	m_listener->processPkg(buf, status, host, port);

	return 0;
}

int CUDPListener::handleWrite()
{
	maskWrite(false);
	return 0;
}

void CUDPListener::handleClose()
{
}

unsigned short CUDPListener::getPort()
{
	return _port;
}
int CUDPListener::sendto(const char* buf, size_t len, std::string const& destip, const int destport)
{
	uint32_t ip;

	if ( isalpha(destip.c_str()[0]))
	{
		hostent *php = gethostbyname(destip.c_str());
		if (NULL == php)
			return -1;
		memcpy(&ip, php->h_addr, sizeof(ip));
	}
	else
		ip = inet_addr(destip.c_str());

	return sendto(buf, len, ntohl(ip), destport);
}

int CUDPListener::sendto(const char* buf, size_t len, uint32_t destip, int destport)
{
	boost::mutex::scoped_lock lock(m_mutex);
	int status;
	struct sockaddr_in destsockaddr;

	memset(&destsockaddr, 0, sizeof(struct sockaddr_in));
	destsockaddr.sin_family = AF_INET;
	destsockaddr.sin_port   = htons(destport);
	destsockaddr.sin_addr.s_addr = htonl(destip);


	status = ::sendto(getHandle(),
			(char *)buf,
			len,
			MSG_DONTWAIT,
			(struct sockaddr *)&destsockaddr,
			(socklen_t)sizeof(struct sockaddr));

	return status;
}
