/*
ported by George Ang<gnap.an<AT>gmail.com>
Copyright (C)2003 Barry Dunne (http://www.emule-project.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <inttypes.h>
#include <assert.h>
#include <time.h>
#include <string>

#ifndef CONTACT_HPP
#define CONTACT_HPP

namespace kademlia
{
	template<typename _uintX, class debugger> class routing_zone;
	template<typename _uintX, class debugger> class kbucket;

	template<typename _uintX>
		class contact 
		{
			public:

				typedef _uintX uint_type;
				typedef contact<uint_type> contact_type;

				friend class routing_zone<uint_type, class debugger>;
				friend class kbucket<uint_type, class debugger>;
		
				contact();

				contact(uint_type const& node_id);

				virtual ~contact();

				const uint_type get_node_id() const;

				void set_node_id(uint_type const& node_id);

				void set_distance(uint_type const& distance);

				uint_type get_distance() const;

				
				std::string get_ip();

				void set_ip(std::string ip);

				int get_port();

				void set_port(int port);

				
				uint8_t get_type() const;

				void update_type();

				void checking_type();

				bool in_use();

				void inc_use();

				void dec_use();

				time_t	get_create_time() const;

				time_t	get_expire_time() const;

				void	set_expire_time(time_t expire_time);

				time_t	get_last_type_set() const;

				bool is_published();

				void set_published();

				virtual bool is_same_contact (contact_type *contact) const;

				virtual bool is_valid();
			private:
				void initialize();

				uint_type 	m_node_id;
				uint_type 	m_distance;
				std::string 	m_node_host;
				int 		m_node_port;
				bool		m_is_published;
				uint8_t		m_type;
				time_t		m_last_type_set;
				time_t		m_expire_time;
				uint32_t	m_in_use;
				time_t		m_create_time;
		};

	template<typename _uintX>
		contact<_uintX>::contact()
		{
		       	initialize(); 
		}

	template<typename _uintX>
		contact<_uintX>::contact(uint_type const& node_id)
		{
			set_node_id(node_id);
			initialize(); 
		}
	template<typename _uintX>
		contact<_uintX>::~contact()
		{
		}

	template<typename _uintX>
		const typename contact<_uintX>::uint_type contact<_uintX>::get_node_id() const
		{
			return m_node_id;
		}

	template<typename _uintX>
		void contact<_uintX>::set_node_id(uint_type const& node_id)
		{
			m_node_id = node_id;
		}

	template<typename _uintX>
		void contact<_uintX>::set_distance(uint_type const& distance)
		{
			m_distance = distance;
		}

	template<typename _uintX>
		class contact<_uintX>::uint_type contact<_uintX>::get_distance() const
		{
			return m_distance;
		}
				

	template<typename _uintX>
		std::string contact<_uintX>::get_ip()
		{
			return m_node_host;
		}

	template<typename _uintX>
		void contact<_uintX>::set_ip(std::string ip)
		{
			m_node_host = ip;
		}

	template<typename _uintX>
		int contact<_uintX>::get_port()
		{
			return m_node_port;
		}

	template<typename _uintX>
		void contact<_uintX>::set_port(int port)
		{
			m_node_port = port;
		}


	template<typename _uintX>
		uint8_t contact<_uintX>::get_type() const
		{
			return m_type;
		}

	template<typename _uintX>
		void contact<_uintX>::update_type()
		{
			uint32_t hours = (time(NULL) - m_create_time)/3600;
			switch(hours)
			{
				case 0:
					m_type = 2;
					m_expire_time = time(NULL) + 3600;
					break;
				case 1:
					m_type = 1;
					m_expire_time = (time_t)(time(NULL) + 5400);
					break;
				default:
					m_type = 0;
					m_expire_time = time(NULL) + 7200;
			}

		}

	template<typename _uintX>
		void contact<_uintX>::checking_type()
		{
			if(time(NULL) - m_last_type_set < 10 || m_type == 4)
				return;
			m_last_type_set = time(NULL);

			m_expire_time = time(NULL) + 120;
			m_type++;

		}

	template<typename _uintX>
		bool contact<_uintX>::in_use()
		{
			return (m_in_use > 0);
		}

	template<typename _uintX>
		void contact<_uintX>::inc_use()
		{
			m_in_use++;
		}

	template<typename _uintX>
		void contact<_uintX>::dec_use()
		{
			if(m_in_use)
				m_in_use--;
			else
				assert(0);

		}					

	template<typename _uintX>
		time_t	contact<_uintX>::get_create_time() const
		{
			return m_create_time;
		}

	template<typename _uintX>
		time_t	contact<_uintX>::get_expire_time() const
		{
			return m_expire_time;
		}

	template<typename _uintX>
		void	contact<_uintX>::set_expire_time(time_t expire_time)
		{
			m_expire_time = expire_time;
		}

	template<typename _uintX>
		time_t	contact<_uintX>::get_last_type_set() const
		{
			return m_last_type_set;
		}

	template<typename _uintX>
		bool contact<_uintX>::is_published()
		{
			return m_is_published;
		}

	template<typename _uintX>
		void contact<_uintX>::set_published()
		{
			m_is_published = true;
		}

	template<typename _uintX>
		bool contact<_uintX>::is_same_contact (contact_type *contact) const
		{
			return m_node_id == contact->get_node_id();
		}

	template<typename _uintX>
		bool contact<_uintX>::is_valid()
		{
			return  m_node_id != 0;
		}

	template<typename _uintX>
		void contact<_uintX>::initialize()
		{
			m_type = 3;
			m_expire_time = 0;
			m_last_type_set = time(NULL);
			m_in_use = 0;
			m_create_time  = time(NULL);
			m_is_published = false;
		}

};
#endif // CONTACT_HPP
