/*
Copyright (C)2006 George Ang (gnap.an AT gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <string>

#ifndef UINT160_HPP
#define UINT160_HPP

class uint160
{
	public:
		const static int digits = 160;
		uint160();
		uint160(bool bitfill);
		uint160(uint32_t value);
		uint160(const char * bytes);
		//Generates a new number, copying the most significant 'count' bits from 'value'.
		//The remaining bits are randomly generated.
		uint160(const uint160 &value, unsigned int count = 160);
		
		const unsigned char* get_data() const;
		unsigned char* c_str();
		//Bit at level 0 being most significant. */
		unsigned char get_8bit_chunk(int val) const;
		uint32_t get_32bit_chunk(int pos) const;
		void set_32bit_chunk(int pos, uint32_t val);
		unsigned int get_bit_number(unsigned int bit) const;
		int compare_to(const uint160 &other) const;
		int compare_to(uint32_t value) const;
		std::string get_hex_string() const;
		std::string get_binary_string(bool trim = false) const;

		uint160& set_value(const uint160 &value);
		uint160& set_value(uint32_t value);
		uint160& set_value_be(const unsigned char *value);
		uint160& set_value_hex(const char *value);
		uint160& set_value_random();
		uint160& set_bit_number(unsigned int bit, unsigned int value);
		uint160& shift_left(unsigned int bits);
		uint160& add(const uint160 &value);
		uint160& add(uint32_t value);
		uint160& subtract(const uint160 &value);
		uint160& subtract(uint32_t value);
		uint160& _xor(const uint160 &value);
		uint160& _xor(const unsigned char *value);
		int log() const;
		void operator+=  (const uint160 &value);
		void operator-=  (const uint160 &value);
		void operator=  (const uint160 &value);
		bool operator<  (const uint160 &value) const;
		bool operator>  (const uint160 &value) const;
		bool operator<= (const uint160 &value) const;
		bool operator>= (const uint160 &value) const;
		bool operator== (const uint160 &value) const;
		bool operator!= (const uint160 &value) const;
		void operator+=  (uint32_t value);
		void operator-=  (uint32_t value);
		void operator=  (uint32_t value);
		bool operator<  (uint32_t value) const;
		bool operator>  (uint32_t value) const;
		bool operator<= (uint32_t value) const;
		bool operator>= (uint32_t value) const;
		bool operator== (uint32_t value) const;
		bool operator!= (uint32_t value) const;
	private:
		unsigned char m_data[20];
};

uint160 get_sha1_hash(std::string const& str);
#endif //UINT160_HPP
